@@ -1,4 +1,4 @@
-import collections
+from datetime import datetime
 
 
 def deep_update(source: dict, overrides: dict):
@@ -26,3 +26,12 @@ def size_to_byte(size: float, unit: str) -> int:
         final_size = size * 1000000000000000
 
     return int(final_size)
+
+
+def datetime_as_milis(date: datetime = datetime.utcnow()) -> int:
+    return int(round(date.timestamp() * 1000))
+
+
+def map_timestamp_file(file_path: str) -> str:
+    path_split = file_path.split('/')
+    return '/'.join(path_split[0:-1]) + '/' + path_split[-1].split('.')[0] + '.ts'
