@@ -1,12 +1,15 @@
 import re
-from distutils.version import LooseVersion
+import traceback
 from threading import Thread
-from typing import Set, List, Tuple, Dict, Iterable
+from typing import Set, List, Tuple, Dict, Iterable, Optional
+
+from packaging.version import parse as parse_version
 
 from bauh.api.abstract.handler import ProcessWatcher
 from bauh.gems.arch import pacman, message, sorting, confirmation
 from bauh.gems.arch.aur import AURClient
 from bauh.gems.arch.exceptions import PackageNotFoundException
+from bauh.gems.arch.util import clean_version
 from bauh.view.util.translation import I18n
 
 
@@ -183,13 +186,26 @@ class DependenciesAnalyser:
 
         if dep_name == dep_exp:
             providers = remote_provided_map.get(dep_name)
+
+            if not providers:  # try to find the package through the pacman's search mechanism
+                match = pacman.find_one_match(dep_name)
+
+                if match:
+                    providers = {match}
+
         else:  # handling cases when the dep has an expression ( e.g: xpto>=0.12 )
             providers = remote_provided_map.get(dep_exp)
 
             if providers is None:
                 providers = remote_provided_map.get(dep_name)
 
-                if providers and len(providers) > 1:
+                if not providers:  # try to find the package through the pacman's search mechanism
+                    match = pacman.find_one_match(dep_name)
+
+                    if match:
+                        providers = {match}
+
+                if providers:
                     no_mapped_data = {p for p in providers if
                                       p not in deps_data}  # checking providers with no mapped data
 
@@ -203,23 +219,26 @@ class DependenciesAnalyser:
 
                     matched_providers = set()
                     split_informed_dep = self.re_dep_operator.split(dep_exp)
-                    version_informed = LooseVersion(split_informed_dep[2])
-                    exp_op = split_informed_dep[1] if split_informed_dep[1] != '=' else '=='
+                    try:
+                        version_required = parse_version(clean_version(split_informed_dep[2]))
+                        exp_op = split_informed_dep[1] if split_informed_dep[1] != '=' else '=='
 
-                    for p in providers:
-                        provided = deps_data[p]['p']
+                        for p in providers:
+                            provided = deps_data[p]['p']
 
-                        for provided_exp in provided:
-                            split_dep = self.re_dep_operator.split(provided_exp)
+                            for provided_exp in provided:
+                                split_dep = self.re_dep_operator.split(provided_exp)
 
-                            if len(split_dep) == 3 and split_dep[0] == dep_name:
-                                provided_version = LooseVersion(split_dep[2])
+                                if len(split_dep) == 3 and split_dep[0] == dep_name:
+                                    version_provided = parse_version(clean_version(split_dep[2]))
 
-                                if eval('provided_version {} version_informed'.format(exp_op)):
-                                    matched_providers.add(p)
-                                    break
+                                    if eval('version_provided {} version_required'.format(exp_op)):
+                                        matched_providers.add(p)
+                                        break
 
-                    providers = matched_providers
+                        providers = matched_providers
+                    except:
+                        traceback.print_exc()
 
         if providers:
             if len(providers) > 1:
@@ -241,14 +260,40 @@ class DependenciesAnalyser:
             missing_deps.add(dep_data)
 
         elif aur_index and dep_name in aur_index:
-            aur_deps.add(dep_name)
-            missing_deps.add((dep_name, 'aur'))
-        else:
-            if watcher:
-                message.show_dep_not_found(dep_name, self.i18n, watcher)
-                raise PackageNotFoundException(dep_name)
+            if dep_name == dep_exp:
+                aur_deps.add(dep_name)
+                missing_deps.add((dep_name, 'aur'))
             else:
-                raise PackageNotFoundException(dep_name)
+                dep_info = self.aur_client.get_info({dep_name})
+
+                if not dep_info:
+                    self.__raise_dependency_not_found(dep_exp, watcher)
+                else:
+                    try:
+                        dep_version = parse_version(clean_version(dep_info[0]['Version']))
+                    except:
+                        traceback.print_exc()
+                        self.__raise_dependency_not_found(dep_exp, watcher)
+
+                    split_informed_dep = self.re_dep_operator.split(dep_exp)
+                    try:
+                        version_required = parse_version(clean_version(split_informed_dep[2]))
+                        exp_op = split_informed_dep[1] if split_informed_dep[1] != '=' else '=='
+
+                        if eval('dep_version {} version_required'.format(exp_op)):
+                            aur_deps.add(dep_name)
+                            missing_deps.add((dep_name, 'aur'))
+                    except:
+                        self.__raise_dependency_not_found(dep_exp, watcher)
+        else:
+            self.__raise_dependency_not_found(dep_exp, watcher)
+
+    def __raise_dependency_not_found(self, dep_exp: str, watcher: Optional[ProcessWatcher]):
+        if watcher:
+            message.show_dep_not_found(dep_exp, self.i18n, watcher)
+            raise PackageNotFoundException(dep_exp)
+        else:
+            raise PackageNotFoundException(dep_exp)
 
     def __fill_aur_update_data(self, pkgname: str, output: dict):
         output[pkgname] = self.aur_client.map_update_data(pkgname, None)
@@ -289,20 +334,20 @@ class DependenciesAnalyser:
                                 version_found = [p for p in provided_map if p.startswith(version_pattern)]
 
                                 if version_found:
-                                    version_found = version_found[0].split('=')[1]
-                                    version_informed = dep_split[2].strip()
+                                    version_found = clean_version(version_found[0].split('=')[1])
+                                    version_required = clean_version(dep_split[2])
 
-                                    if ':' not in version_informed:
-                                        version_found = version_found.split(':')[-1]
+                                    try:
+                                        version_found = parse_version(version_found)
+                                        version_required = parse_version(version_required)
 
-                                    if '-' not in version_informed:
-                                        version_found = version_found.split('-')[0]
+                                        op = dep_split[1] if dep_split[1] != '=' else '=='
+                                        match = eval('version_found {} version_required'.format(op))
+                                    except:
+                                        match = False
+                                        traceback.print_exc()
 
-                                    version_found = LooseVersion(version_found)
-                                    version_informed = LooseVersion(version_informed)
-
-                                    op = dep_split[1] if dep_split[1] != '=' else '=='
-                                    if not eval('version_found {} version_informed'.format(op)):
+                                    if not match:
                                         self._fill_missing_dep(dep_name=dep_name, dep_exp=dep, aur_index=aur_index,
                                                                missing_deps=missing_deps,
                                                                remote_provided_map=remote_provided_map,
@@ -348,7 +393,7 @@ class DependenciesAnalyser:
                 for t in aur_threads:
                     t.join()
 
-            missing_subdeps = self.map_missing_deps(pkgs_data=deps_data, provided_map=provided_map, aur_index=aur_index,
+            missing_subdeps = self.map_missing_deps(pkgs_data={**deps_data}, provided_map=provided_map, aur_index=aur_index,
                                                     deps_checked=deps_checked, sort=False, deps_data=deps_data,
                                                     watcher=watcher,
                                                     remote_provided_map=remote_provided_map,
@@ -456,7 +501,9 @@ class DependenciesAnalyser:
         return missing_deps
 
     def map_all_required_by(self, pkgnames: Iterable[str], to_ignore: Set[str]) -> Set[str]:
-        to_ignore.update(pkgnames)
+        if pkgnames:
+            to_ignore.update(pkgnames)
+
         all_requirements = {req for reqs in pacman.map_required_by(pkgnames).values() for req in reqs if req not in to_ignore}
 
         if all_requirements:
