@@ -4,17 +4,21 @@ import os
 import re
 import time
 import traceback
+from datetime import datetime, timedelta
 from pathlib import Path
 from threading import Thread
+from typing import Optional
 
 import requests
 
 from bauh.api.abstract.context import ApplicationContext
 from bauh.api.abstract.handler import TaskManager
+from bauh.commons import system
+from bauh.commons.boot import CreateConfigFile
 from bauh.commons.html import bold
-from bauh.commons.system import run_cmd, new_root_subprocess, ProcessHandler
-from bauh.gems.arch import pacman, disk, CUSTOM_MAKEPKG_FILE, CONFIG_DIR, BUILD_DIR, \
-    AUR_INDEX_FILE, get_icon_path, database, mirrors, ARCH_CACHE_PATH
+from bauh.commons.system import new_root_subprocess, ProcessHandler
+from bauh.gems.arch import pacman, disk, CUSTOM_MAKEPKG_FILE, CONFIG_DIR, AUR_INDEX_FILE, get_icon_path, database, \
+    mirrors, ARCH_CACHE_PATH, AUR_INDEX_TS_FILE, aur
 from bauh.gems.arch.aur import URL_INDEX
 from bauh.view.util.translation import I18n
 
@@ -28,94 +32,181 @@ RE_CLEAR_REPLACE = re.compile(r'[\-_.]')
 
 class AURIndexUpdater(Thread):
 
-    def __init__(self, context: ApplicationContext):
+    def __init__(self, context: ApplicationContext, taskman: TaskManager, create_config: Optional[CreateConfigFile] = None, arch_config: Optional[dict] = None):
         super(AURIndexUpdater, self).__init__(daemon=True)
         self.http_client = context.http_client
         self.i18n = context.i18n
         self.logger = context.logger
+        self.taskman = taskman
+        self.task_id = 'index_aur'
+        self.create_config = create_config
+        self.config = arch_config
+        self.taskman.register_task(self.task_id, self.i18n['arch.task.aur.index.status'], get_icon_path())
+
+    def should_update(self) -> bool:
+        if not aur.is_supported(self.config):
+            return False
 
-    def run(self):
-        self.logger.info('Pre-indexing AUR packages')
         try:
+            exp_hours = int(self.config['aur_idx_exp'])
+        except:
+            traceback.print_exc()
+            return True
+
+        if exp_hours <= 0:
+            return True
+
+        if not os.path.exists(AUR_INDEX_FILE):
+            return True
+
+        if not os.path.exists(AUR_INDEX_TS_FILE):
+            return True
+
+        with open(AUR_INDEX_TS_FILE) as f:
+            timestamp_str = f.read()
+
+        try:
+            index_timestamp = datetime.fromtimestamp(float(timestamp_str))
+            return (index_timestamp + timedelta(hours=exp_hours)) <= datetime.utcnow()
+        except:
+            traceback.print_exc()
+            return True
+
+    def update_index(self):
+        self.logger.info('Indexing AUR packages')
+        self.taskman.update_progress(self.task_id, 5, self.i18n['arch.task.aur.index.substatus.download'])
+        try:
+            index_ts = datetime.utcnow().timestamp()
             res = self.http_client.get(URL_INDEX)
 
             if res and res.text:
+                index_progress = 50
+                self.taskman.update_progress(self.task_id, index_progress,
+                                             self.i18n['arch.task.aur.index.substatus.gen_index'])
                 indexed = 0
-                Path(BUILD_DIR).mkdir(parents=True, exist_ok=True)
+
+                Path(os.path.dirname(AUR_INDEX_FILE)).mkdir(parents=True, exist_ok=True)
 
                 with open(AUR_INDEX_FILE, 'w+') as f:
-                    for n in res.text.split('\n'):
+                    lines = res.text.split('\n')
+                    progress_inc = round(len(lines) / 50)  # 1%
+
+                    perc_count = 0
+                    for n in lines:
+                        if index_progress < 100 and perc_count == progress_inc:
+                            index_progress += 1
+                            perc_count = 0
+                            self.taskman.update_progress(self.task_id, index_progress,
+                                                         self.i18n['arch.task.aur.index.substatus.gen_index'])
+
                         if n and not n.startswith('#'):
                             f.write('{}={}\n'.format(RE_CLEAR_REPLACE.sub('', n), n))
                             indexed += 1
 
+                        perc_count += 1
+
+                with open(AUR_INDEX_TS_FILE, 'w+') as f:
+                    f.write(str(index_ts))
+
                 self.logger.info('Pre-indexed {} AUR package names at {}'.format(indexed, AUR_INDEX_FILE))
+                self.taskman.update_progress(self.task_id, 100, None)
+
             else:
                 self.logger.warning('No data returned from: {}'.format(URL_INDEX))
+                self.taskman.update_progress(self.task_id, 100, self.i18n['arch.task.aur.index.substatus.error.no_data'])
+
         except requests.exceptions.ConnectionError:
             self.logger.warning('No internet connection: could not pre-index packages')
+            self.taskman.update_progress(self.task_id, 100, self.i18n['arch.task.aur.index.substatus.error.download'])
 
-        self.logger.info("Finished")
+    def run(self):
+        ti = time.time()
+
+        if self.create_config:
+            self.taskman.update_progress(self.task_id, 0, self.i18n['task.waiting_task'].format(self.create_config.task_name))
+            self.create_config.join()
+            self.config = self.create_config.config
+
+        self.taskman.update_progress(self.task_id, 1, self.i18n['arch.task.aur.index.substatus.checking'])
+
+        if self.should_update():
+            self.update_index()
+        else:
+            self.logger.info("AUR index is up to date. Aborting...")
+            self.taskman.update_progress(self.task_id, 100, None)
+
+        tf = time.time()
+        self.taskman.finish_task(self.task_id)
+        self.logger.info("Finished. Took {0:.5f} seconds".format(tf - ti))
 
 
 class ArchDiskCacheUpdater(Thread):
 
-    def __init__(self, task_man: TaskManager, arch_config: dict, i18n: I18n, logger: logging.Logger, controller: "ArchManager", internet_available: bool):
+    def __init__(self, taskman: TaskManager, i18n: I18n, logger: logging.Logger,
+                 controller: "ArchManager", internet_available: bool, aur_indexer: Thread,
+                 create_config: CreateConfigFile):
         super(ArchDiskCacheUpdater, self).__init__(daemon=True)
         self.logger = logger
-        self.task_man = task_man
+        self.taskman = taskman
         self.task_id = 'arch_cache_up'
         self.i18n = i18n
-        self.prepared = 0
-        self.prepared_template = self.i18n['arch.task.disk_cache.prepared'] + ': {}/ {}'
         self.indexed = 0
         self.indexed_template = self.i18n['arch.task.disk_cache.indexed'] + ': {}/ {}'
         self.to_index = 0
         self.progress = 0  # progress is defined by the number of packages prepared and indexed
-        self.repositories = arch_config['repositories']
-        self.aur = bool(arch_config['aur'])
         self.controller = controller
         self.internet_available = internet_available
         self.installed_hash_path = '{}/installed.sha1'.format(ARCH_CACHE_PATH)
         self.installed_cache_dir = '{}/installed'.format(ARCH_CACHE_PATH)
-
-    def update_prepared(self, pkgname: str, add: bool = True):
-        if add:
-            self.prepared += 1
-
-        sub = self.prepared_template.format(self.prepared, self.to_index)
-        progress = ((self.prepared + self.indexed) / self.progress) * 100 if self.progress > 0 else 0
-        self.task_man.update_progress(self.task_id, progress, sub)
+        self.aur_indexer = aur_indexer
+        self.create_config = create_config
+        self.taskman.register_task(self.task_id, self.i18n['arch.task.disk_cache'], get_icon_path())
 
     def update_indexed(self, pkgname: str):
         self.indexed += 1
         sub = self.indexed_template.format(self.indexed, self.to_index)
-        progress = ((self.prepared + self.indexed) / self.progress) * 100 if self.progress > 0 else 0
-        self.task_man.update_progress(self.task_id, progress, sub)
+        progress = self.progress + (self.indexed / self.to_index) * 50
+        self.taskman.update_progress(self.task_id, progress, sub)
+
+    def _update_progress(self, progress: float, msg: str):
+        self.progress = progress
+        self.taskman.update_progress(self.task_id, self.progress, msg)
+
+    def _notify_reading_files(self):
+        self._update_progress(50, self.i18n['arch.task.disk_cache.indexing'])
 
     def run(self):
-        if not any([self.aur, self.repositories]):
+        ti = time.time()
+        self.taskman.update_progress(self.task_id, 0, self.i18n['task.waiting_task'].format(self.create_config.task_name))
+        self.create_config.join()
+
+        config = self.create_config.config
+        aur_supported, repositories = aur.is_supported(config), config['repositories']
+
+        self.taskman.update_progress(self.task_id, 1, None)
+        if not any([aur_supported, repositories]):
+            self.taskman.update_progress(self.task_id, 100, self.i18n['arch.task.disabled'])
+            self.taskman.finish_task(self.task_id)
             return
 
-        ti = time.time()
-        self.task_man.register_task(self.task_id, self.i18n['arch.task.disk_cache'], get_icon_path())
-
-        self.task_man.update_progress(self.task_id, 1, '')
-
         self.logger.info("Checking already cached package data")
+        self._update_progress(1, self.i18n['arch.task.disk_cache.checking'])
 
         cache_dirs = [fpath for fpath in glob.glob('{}/*'.format(self.installed_cache_dir)) if os.path.isdir(fpath)]
 
         not_cached_names = None
 
+        self._update_progress(15, self.i18n['arch.task.disk_cache.checking'])
         if cache_dirs:  # if there are cache data
             installed_names = pacman.list_installed_names()
             cached_pkgs = {cache_dir.split('/')[-1] for cache_dir in cache_dirs}
 
             not_cached_names = installed_names.difference(cached_pkgs)
+            self._update_progress(20, self.i18n['arch.task.disk_cache.checking'])
+
             if not not_cached_names:
-                self.task_man.update_progress(self.task_id, 100, '')
-                self.task_man.finish_task(self.task_id)
+                self.taskman.update_progress(self.task_id, 100, '')
+                self.taskman.finish_task(self.task_id)
                 tf = time.time()
                 time_msg = '{0:.2f} seconds'.format(tf - ti)
                 self.logger.info('Finished: no package data to cache ({})'.format(time_msg))
@@ -123,23 +214,29 @@ class ArchDiskCacheUpdater(Thread):
 
         self.logger.info('Pre-caching installed Arch packages data to disk')
 
+        if aur_supported and self.aur_indexer:
+            self.taskman.update_progress(self.task_id, 20, self.i18n['arch.task.disk_cache.waiting_aur_index'].format(bold(self.i18n['arch.task.aur.index.status'])))
+            self.aur_indexer.join()
+
+        self._update_progress(21, self.i18n['arch.task.disk_cache.checking'])
         installed = self.controller.read_installed(disk_loader=None, internet_available=self.internet_available,
                                                    only_apps=False, pkg_types=None, limit=-1, names=not_cached_names,
                                                    wait_disk_cache=False).installed
 
-        self.task_man.update_progress(self.task_id, 0, self.i18n['arch.task.disk_cache.reading'])
+        self._update_progress(35, self.i18n['arch.task.disk_cache.checking'])
 
         saved = 0
-        pkgs = {p.name: p for p in installed if ((self.aur and p.repository == 'aur') or (self.repositories and p.repository != 'aur')) and not os.path.exists(p.get_disk_cache_path())}
 
+        pkgs = {p.name: p for p in installed if ((aur_supported and p.repository == 'aur') or (repositories and p.repository != 'aur')) and not os.path.exists(p.get_disk_cache_path())}
         self.to_index = len(pkgs)
-        self.progress = self.to_index * 2
-        self.update_prepared(None, add=False)
 
         # overwrite == True because the verification already happened
-        saved += disk.save_several(pkgs, when_prepared=self.update_prepared, after_written=self.update_indexed, overwrite=True)
-        self.task_man.update_progress(self.task_id, 100, None)
-        self.task_man.finish_task(self.task_id)
+        self._update_progress(40, self.i18n['arch.task.disk_cache.reading_files'])
+        saved += disk.write_several(pkgs=pkgs,
+                                    after_desktop_files=self._notify_reading_files,
+                                    after_written=self.update_indexed, overwrite=True)
+        self.taskman.update_progress(self.task_id, 100, None)
+        self.taskman.finish_task(self.task_id)
 
         tf = time.time()
         time_msg = '{0:.2f} seconds'.format(tf - ti)
@@ -148,7 +245,7 @@ class ArchDiskCacheUpdater(Thread):
 
 class ArchCompilationOptimizer(Thread):
 
-    def __init__(self, arch_config: dict, i18n: I18n, logger: logging.Logger, task_man: TaskManager = None):
+    def __init__(self, i18n: I18n, logger: logging.Logger, taskman: TaskManager, create_config: Optional[CreateConfigFile] = None):
         super(ArchCompilationOptimizer, self).__init__(daemon=True)
         self.logger = logger
         self.i18n = i18n
@@ -156,19 +253,14 @@ class ArchCompilationOptimizer(Thread):
         self.re_compress_zst = re.compile(r'#?\s*COMPRESSZST\s*=\s*.+')
         self.re_build_env = re.compile(r'\s+BUILDENV\s*=.+')
         self.re_ccache = re.compile(r'!?ccache')
-        self.task_man = task_man
+        self.taskman = taskman
         self.task_id = 'arch_make_optm'
-        self.optimizations = bool(arch_config['optimize'])
+        self.create_config = create_config
+        self.taskman.register_task(self.task_id, self.i18n['arch.task.optimizing'].format(bold('makepkg.conf')), get_icon_path())
 
     def _is_ccache_installed(self) -> bool:
-        return bool(run_cmd('which ccache', print_error=False))
-
-    def _update_progress(self, progress: float, substatus: str = None):
-        if self.task_man:
-            self.task_man.update_progress(self.task_id, progress, substatus)
-
-            if progress == 100:
-                self.task_man.finish_task(self.task_id)
+        code, _ = system.execute(cmd='which ccache', output=False)
+        return code == 0
 
     def optimize(self):
         ti = time.time()
@@ -202,7 +294,7 @@ class ArchCompilationOptimizer(Thread):
                 else:
                     optimizations.append('MAKEFLAGS="-j$(nproc)"')
 
-            self._update_progress(20)
+            self.taskman.update_progress(self.task_id, 20, None)
 
             compress_xz = self.re_compress_xz.findall(custom_makepkg or global_makepkg)
 
@@ -217,7 +309,7 @@ class ArchCompilationOptimizer(Thread):
             else:
                 optimizations.append('COMPRESSXZ=(xz -c -z - --threads=0)')
 
-            self._update_progress(40)
+            self.taskman.update_progress(self.task_id, 40, None)
 
             compress_zst = self.re_compress_zst.findall(custom_makepkg or global_makepkg)
 
@@ -232,7 +324,7 @@ class ArchCompilationOptimizer(Thread):
             else:
                 optimizations.append('COMPRESSZST=(zstd -c -z -q - --threads=0)')
 
-            self._update_progress(60)
+            self.taskman.update_progress(self.task_id, 60, None)
 
             build_envs = self.re_build_env.findall(custom_makepkg or global_makepkg)
 
@@ -262,7 +354,7 @@ class ArchCompilationOptimizer(Thread):
                     self.logger.info('Adding a BUILDENV declaration')
                     optimizations.append('BUILDENV=(ccache)')
 
-            self._update_progress(80)
+            self.taskman.update_progress(self.task_id, 80, None)
 
             if custom_makepkg and optimizations:
                 generated_by = '# <generated by bauh>\n'
@@ -279,43 +371,91 @@ class ArchCompilationOptimizer(Thread):
                     self.logger.info("Removing old optimized 'makepkg.conf' at '{}'".format(CUSTOM_MAKEPKG_FILE))
                     os.remove(CUSTOM_MAKEPKG_FILE)
 
+            self.taskman.update_progress(self.task_id, 100, None)
             tf = time.time()
-            self._update_progress(100)
-            self.logger.info("Optimizations took {0:.2f} seconds".format(tf - ti))
-            self.logger.info('Finished')
+            self.logger.info('Finished. {0:.2f} seconds'.format(tf - ti))
 
     def run(self):
-        if not self.optimizations:
-            self.logger.info("Arch packages compilation optimizations are disabled")
+        ti = time.time()
+        if self.create_config:
+            self.taskman.update_progress(self.task_id, 0, self.i18n['task.waiting_task'].format(bold(self.create_config.task_name)))
+            self.create_config.join()
 
-            if os.path.exists(CUSTOM_MAKEPKG_FILE):
-                self.logger.info("Removing custom 'makepkg.conf' -> '{}'".format(CUSTOM_MAKEPKG_FILE))
-                os.remove(CUSTOM_MAKEPKG_FILE)
+            self.taskman.update_progress(self.task_id, 1, None)
 
-            self.logger.info('Finished')
-        else:
-            if self.task_man:
-                self.task_man.register_task(self.task_id, self.i18n['arch.task.optimizing'].format(bold('makepkg.conf')), get_icon_path())
+            if self.create_config.config['optimize'] and aur.is_supported(self.create_config.config):
+                try:
+                    self.optimize()
+                except:
+                    self.logger.error("Unexpected exception")
+                    traceback.print_exc()
+                    self.taskman.update_progress(self.task_id, 100, None)
+            else:
+                self.logger.info("AUR packages compilation optimizations are disabled")
 
-            self.optimize()
+                if os.path.exists(CUSTOM_MAKEPKG_FILE):
+                    try:
+                        self.logger.info("Removing custom 'makepkg.conf' -> '{}'".format(CUSTOM_MAKEPKG_FILE))
+                        os.remove(CUSTOM_MAKEPKG_FILE)
+                    except:
+                        self.logger.error("Unexpected exception")
+                        traceback.print_exc()
+
+                self.taskman.update_progress(self.task_id, 100, self.i18n['arch.task.disabled'])
+
+        tf = time.time()
+        self.taskman.finish_task(self.task_id)
+        self.logger.info('Finished. Took {0:.2f} seconds'.format(tf - ti))
 
 
 class RefreshMirrors(Thread):
 
-    def __init__(self, taskman: TaskManager, root_password: str, i18n: I18n, sort_limit: int, logger: logging.Logger):
+    def __init__(self, taskman: TaskManager, root_password: str, i18n: I18n, logger: logging.Logger,
+                 create_config: CreateConfigFile):
         super(RefreshMirrors, self).__init__(daemon=True)
         self.taskman = taskman
         self.i18n = i18n
         self.logger = logger
         self.root_password = root_password
         self.task_id = "arch_mirrors"
-        self.sort_limit = sort_limit
+        self.create_config = create_config
+        self.refreshed = False
+        self.task_name = self.i18n['arch.task.mirrors']
+        self.taskman.register_task(self.task_id, self.task_name, get_icon_path())
 
     def _notify_output(self, output: str):
         self.taskman.update_output(self.task_id, output)
 
+    @staticmethod
+    def is_enabled(arch_config: dict, aur_supported: bool) -> bool:
+        return (arch_config['repositories'] or aur_supported) \
+               and arch_config['refresh_mirrors_startup'] and pacman.is_mirrors_available()
+
+    @classmethod
+    def should_synchronize(cls, arch_config: dict, aur_supported: bool, logger: logging.Logger) -> bool:
+        return cls.is_enabled(arch_config, aur_supported) and mirrors.should_sync(logger)
+
     def run(self):
-        self.taskman.register_task(self.task_id, self.i18n['arch.task.mirrors'], get_icon_path())
+        ti = time.time()
+        self.taskman.update_progress(self.task_id, 0, self.i18n['task.waiting_task'].format(bold(self.create_config.task_name)))
+        self.create_config.join()
+
+        arch_config = self.create_config.config
+        aur_supported = aur.is_supported(arch_config)
+
+        self.taskman.update_progress(self.task_id, 1, self.i18n['arch.task.checking_settings'])
+
+        if not self.is_enabled(arch_config, aur_supported):
+            self.taskman.update_progress(self.task_id, 100, self.i18n['arch.task.disabled'])
+            self.taskman.finish_task(self.task_id)
+            return
+
+        if not mirrors.should_sync(self.logger):
+            self.taskman.update_progress(self.task_id, 100, self.i18n['arch.task.mirrors.cached'])
+            self.taskman.finish_task(self.task_id)
+            return
+
+        sort_limit = arch_config['mirrors_sort_limit']
         self.logger.info("Refreshing mirrors")
 
         handler = ProcessHandler()
@@ -325,15 +465,16 @@ class RefreshMirrors(Thread):
 
             if success:
 
-                if self.sort_limit is not None and self.sort_limit >= 0:
+                if sort_limit is not None and sort_limit >= 0:
                     self.taskman.update_progress(self.task_id, 50, self.i18n['arch.custom_action.refresh_mirrors.status.updating'])
                     try:
-                        handler.handle_simple(pacman.sort_fastest_mirrors(self.root_password, self.sort_limit), output_handler=self._notify_output)
+                        handler.handle_simple(pacman.sort_fastest_mirrors(self.root_password, sort_limit), output_handler=self._notify_output)
                     except:
                         self.logger.error("Could not sort mirrors by speed")
                         traceback.print_exc()
 
                 mirrors.register_sync(self.logger)
+                self.refreshed = True
             else:
                 self.logger.error("It was not possible to refresh mirrors")
         except:
@@ -342,12 +483,14 @@ class RefreshMirrors(Thread):
 
         self.taskman.update_progress(self.task_id, 100, None)
         self.taskman.finish_task(self.task_id)
-        self.logger.info("Finished")
+        tf = time.time()
+        self.logger.info("Finished. Took {0:.2f} seconds".format(tf - ti))
 
 
 class SyncDatabases(Thread):
 
-    def __init__(self, taskman: TaskManager, root_password: str, i18n: I18n, logger: logging.Logger, refresh_mirrors: RefreshMirrors = None):
+    def __init__(self, taskman: TaskManager, root_password: str, i18n: I18n, logger: logging.Logger,
+                 refresh_mirrors: RefreshMirrors, create_config: CreateConfigFile):
         super(SyncDatabases, self).__init__(daemon=True)
         self.task_man = taskman
         self.i18n = i18n
@@ -356,15 +499,46 @@ class SyncDatabases(Thread):
         self.root_password = root_password
         self.refresh_mirrors = refresh_mirrors
         self.logger = logger
+        self.create_config = create_config
+        self.task_name = self.i18n['arch.sync_databases.substatus']
+        self.taskman.register_task(self.task_id, self.task_name, get_icon_path())
+        self.synchronized = False
+
+    @staticmethod
+    def is_enabled(arch_config: dict, aur_supported: bool) -> bool:
+        return arch_config['sync_databases_startup'] and (aur_supported or arch_config['repositories'])
+
+    @classmethod
+    def should_sync(cls, mirrors_refreshed: bool, arch_config: dict, aur_supported: bool, logger: logging.Logger):
+        return mirrors_refreshed or (cls.is_enabled(arch_config, aur_supported) and database.should_sync(arch_config, aur_supported, None, logger))
 
     def run(self) -> None:
+        self.taskman.update_progress(self.task_id, 0, self.i18n['task.waiting_task'].format(bold(self.create_config.task_name)))
+        self.create_config.join()
+
+        self.taskman.update_progress(self.task_id, 0, self.i18n['task.waiting_task'].format(bold(self.refresh_mirrors.task_name)))
+        self.refresh_mirrors.join()
+
+        self.taskman.update_progress(self.task_id, 1,  self.i18n['arch.task.checking_settings'])
+
+        arch_config = self.create_config.config
+        aur_supported = aur.is_supported(arch_config)
+        if not self.is_enabled(arch_config, aur_supported):
+            self.taskman.update_progress(self.task_id, 100, self.i18n['arch.task.disabled'])
+            self.taskman.finish_task(self.task_id)
+            return
+
+        shoud_sync = self.refresh_mirrors.refreshed or (database.should_sync(arch_config, aur_supported, None, self.logger))
+
+        if not shoud_sync:
+            self.taskman.update_progress(self.task_id, 100, self.i18n['arch.sync_databases.substatus.synchronized'])
+            self.taskman.finish_task(self.task_id)
+            self.synchronized = True
+            return
+
         self.logger.info("Synchronizing databases")
         self.taskman.register_task(self.task_id, self.i18n['arch.sync_databases.substatus'], get_icon_path())
 
-        if self.refresh_mirrors and self.refresh_mirrors.is_alive():
-            self.taskman.update_progress(self.task_id, 0, self.i18n['arch.task.sync_databases.waiting'].format('"{}"'.format(self.i18n['arch.task.mirrors'])))
-            self.refresh_mirrors.join()
-
         progress = 10
         dbs = pacman.get_databases()
         self.taskman.update_progress(self.task_id, progress, None)
@@ -403,6 +577,7 @@ class SyncDatabases(Thread):
 
                 if p.returncode == 0:
                     database.register_sync(self.logger)
+                    self.synchronized = True
                 else:
                     self.logger.error("Could not synchronize database")
 
