@@ -1,30 +1,35 @@
 import os
+import re
 import traceback
 from datetime import datetime
 from math import floor
 from pathlib import Path
 from threading import Thread
-from typing import List, Set, Type, Tuple
+from typing import List, Set, Type, Tuple, Optional
+
+from packaging.version import Version
 
 from bauh.api.abstract.controller import SearchResult, SoftwareManager, ApplicationContext, UpgradeRequirements, \
-    UpgradeRequirement, TransactionResult
+    UpgradeRequirement, TransactionResult, SoftwareAction
 from bauh.api.abstract.disk import DiskCacheLoader
 from bauh.api.abstract.handler import ProcessWatcher, TaskManager
 from bauh.api.abstract.model import PackageHistory, PackageUpdate, SoftwarePackage, PackageSuggestion, \
     SuggestionPriority, PackageStatus
 from bauh.api.abstract.view import MessageType, FormComponent, SingleSelectComponent, InputOption, SelectViewType, \
     ViewComponent, PanelComponent
-from bauh.commons import user, internet
-from bauh.commons.config import save_config
+from bauh.commons import user
+from bauh.commons.boot import CreateConfigFile
 from bauh.commons.html import strip_html, bold
-from bauh.commons.system import SystemProcess, ProcessHandler
-from bauh.gems.flatpak import flatpak, SUGGESTIONS_FILE, CONFIG_FILE, UPDATES_IGNORED_FILE, CONFIG_DIR
-from bauh.gems.flatpak.config import read_config
+from bauh.commons.system import ProcessHandler
+from bauh.gems.flatpak import flatpak, SUGGESTIONS_FILE, CONFIG_FILE, UPDATES_IGNORED_FILE, CONFIG_DIR, EXPORTS_PATH, \
+    get_icon_path, VERSION_1_5, VERSION_1_4
+from bauh.gems.flatpak.config import FlatpakConfigManager
 from bauh.gems.flatpak.constants import FLATHUB_API_URL
 from bauh.gems.flatpak.model import FlatpakApplication
 from bauh.gems.flatpak.worker import FlatpakAsyncDataLoader, FlatpakUpdateLoader
 
 DATE_FORMAT = '%Y-%m-%dT%H:%M:%S.000Z'
+RE_INSTALL_REFS = re.compile(r'\d+\)\s+(.+)')
 
 
 class FlatpakManager(SoftwareManager):
@@ -32,13 +37,14 @@ class FlatpakManager(SoftwareManager):
     def __init__(self, context: ApplicationContext):
         super(FlatpakManager, self).__init__(context=context)
         self.i18n = context.i18n
-        self.api_cache = context.cache_factory.new()
-        self.category_cache = context.cache_factory.new()
+        self.api_cache = context.cache_factory.new(None)
+        self.category_cache = context.cache_factory.new(None)
         context.disk_loader_factory.map(FlatpakApplication, self.api_cache)
         self.enabled = True
         self.http_client = context.http_client
-        self.suggestions_cache = context.cache_factory.new()
+        self.suggestions_cache = context.cache_factory.new(None)
         self.logger = context.logger
+        self.configman = FlatpakConfigManager()
 
     def get_managed_types(self) -> Set["type"]:
         return {FlatpakApplication}
@@ -107,10 +113,10 @@ class FlatpakManager(SoftwareManager):
         res.total = len(res.installed) + len(res.new)
         return res
 
-    def _add_updates(self, version: str, output: list):
+    def _add_updates(self, version: Version, output: list):
         output.append(flatpak.list_updates_as_str(version))
 
-    def read_installed(self, disk_loader: DiskCacheLoader, limit: int = -1, only_apps: bool = False, pkg_types: Set[Type[SoftwarePackage]] = None, internet_available: bool = None) -> SearchResult:
+    def read_installed(self, disk_loader: Optional[DiskCacheLoader], limit: int = -1, only_apps: bool = False, pkg_types: Set[Type[SoftwarePackage]] = None, internet_available: bool = None) -> SearchResult:
         version = flatpak.get_version()
 
         updates = []
@@ -137,7 +143,7 @@ class FlatpakManager(SoftwareManager):
                 models.append(model)
 
                 if update_map and (update_map['full'] or update_map['partial']):
-                    if version >= '1.4.0':
+                    if version >= VERSION_1_4:
                         update_id = '{}/{}/{}'.format(app_json['id'], app_json['branch'], app_json['installation'])
 
                         if update_map['full'] and update_id in update_map['full']:
@@ -166,29 +172,28 @@ class FlatpakManager(SoftwareManager):
         return SearchResult(models, None, len(models))
 
     def downgrade(self, pkg: FlatpakApplication, root_password: str, watcher: ProcessWatcher) -> bool:
-        handler = ProcessHandler(watcher)
-        pkg.commit = flatpak.get_commit(pkg.id, pkg.branch, pkg.installation)
+        if not self._make_exports_dir(watcher):
+            return False
 
         watcher.change_progress(10)
         watcher.change_substatus(self.i18n['flatpak.downgrade.commits'])
-        commits = flatpak.get_app_commits(pkg.ref, pkg.origin, pkg.installation, handler)
 
-        if commits is None:
-            return False
-
-        commit_idx = commits.index(pkg.commit)
+        history = self.get_history(pkg, full_commit_str=True)
 
         # downgrade is not possible if the app current commit in the first one:
-        if commit_idx == len(commits) - 1:
-            watcher.show_message(self.i18n['flatpak.downgrade.impossible.title'], self.i18n['flatpak.downgrade.impossible.body'], MessageType.WARNING)
+        if history.pkg_status_idx == len(history.history) - 1:
+            watcher.show_message(self.i18n['flatpak.downgrade.impossible.title'],
+                                 self.i18n['flatpak.downgrade.impossible.body'].format(bold(pkg.name)),
+                                 MessageType.ERROR)
             return False
 
-        commit = commits[commit_idx + 1]
+        commit = history.history[history.pkg_status_idx + 1]['commit']
         watcher.change_substatus(self.i18n['flatpak.downgrade.reverting'])
         watcher.change_progress(50)
-        success = handler.handle(SystemProcess(subproc=flatpak.downgrade(pkg.ref, commit, pkg.installation, root_password),
-                                               success_phrases=['Changes complete.', 'Updates complete.'],
-                                               wrong_error_phrase='Warning'))
+        success, _ = ProcessHandler(watcher).handle_simple(flatpak.downgrade(pkg.ref,
+                                                                             commit,
+                                                                             pkg.installation,
+                                                                             root_password))
         watcher.change_progress(100)
         return success
 
@@ -198,20 +203,24 @@ class FlatpakManager(SoftwareManager):
 
     def upgrade(self, requirements: UpgradeRequirements, root_password: str, watcher: ProcessWatcher) -> bool:
         flatpak_version = flatpak.get_version()
+
+        if not self._make_exports_dir(watcher):
+            return False
+
         for req in requirements.to_upgrade:
             watcher.change_status("{} {} ({})...".format(self.i18n['manage_window.status.upgrading'], req.pkg.name, req.pkg.version))
             related, deps = False, False
             ref = req.pkg.ref
 
-            if req.pkg.partial and flatpak_version < '1.5':
+            if req.pkg.partial and flatpak_version < VERSION_1_5:
                 related, deps = True, True
                 ref = req.pkg.base_ref
 
             try:
-                res = ProcessHandler(watcher).handle(SystemProcess(subproc=flatpak.update(app_ref=ref,
-                                                                                          installation=req.pkg.installation,
-                                                                                          related=related,
-                                                                                          deps=deps)))
+                res, _ = ProcessHandler(watcher).handle_simple(flatpak.update(app_ref=ref,
+                                                                              installation=req.pkg.installation,
+                                                                              related=related,
+                                                                              deps=deps))
 
                 watcher.change_substatus('')
                 if not res:
@@ -227,7 +236,11 @@ class FlatpakManager(SoftwareManager):
         return True
 
     def uninstall(self, pkg: FlatpakApplication, root_password: str, watcher: ProcessWatcher, disk_loader: DiskCacheLoader) -> TransactionResult:
-        uninstalled = ProcessHandler(watcher).handle(SystemProcess(subproc=flatpak.uninstall(pkg.ref, pkg.installation)))
+
+        if not self._make_exports_dir(watcher):
+            return TransactionResult.fail()
+
+        uninstalled, _ = ProcessHandler(watcher).handle_simple(flatpak.uninstall(pkg.ref, pkg.installation))
 
         if uninstalled:
             if self.suggestions_cache:
@@ -241,10 +254,10 @@ class FlatpakManager(SoftwareManager):
     def get_info(self, app: FlatpakApplication) -> dict:
         if app.installed:
             version = flatpak.get_version()
-            id_ = app.base_id if app.partial and version < '1.5' else app.id
+            id_ = app.base_id if app.partial and version < VERSION_1_5 else app.id
             app_info = flatpak.get_app_info_fields(id_, app.branch, app.installation)
 
-            if app.partial and version < '1.5':
+            if app.partial and version < VERSION_1_5:
                 app_info['id'] = app.id
                 app_info['ref'] = app.ref
 
@@ -286,22 +299,51 @@ class FlatpakManager(SoftwareManager):
             else:
                 return {}
 
-    def get_history(self, pkg: FlatpakApplication) -> PackageHistory:
+    def get_history(self, pkg: FlatpakApplication, full_commit_str: bool = False) -> PackageHistory:
         pkg.commit = flatpak.get_commit(pkg.id, pkg.branch, pkg.installation)
-        commits = flatpak.get_app_commits_data(pkg.ref, pkg.origin, pkg.installation)
-        status_idx = 0
+        pkg_commit = pkg.commit if pkg.commit else None
 
-        for idx, data in enumerate(commits):
-            if data['commit'] == pkg.commit:
-                status_idx = idx
-                break
+        if pkg_commit and not full_commit_str:
+            pkg_commit = pkg_commit[0:8]
+
+        commits = flatpak.get_app_commits_data(pkg.ref, pkg.origin, pkg.installation, full_str=full_commit_str)
+
+        status_idx = 0
+        commit_found = False
+
+        if pkg_commit is None and len(commits) > 1 and commits[0]['commit'] == '(null)':
+            del commits[0]
+            pkg_commit = commits[0]
+            commit_found = True
+
+        if not commit_found:
+            for idx, data in enumerate(commits):
+                if data['commit'] == pkg_commit:
+                    status_idx = idx
+                    commit_found = True
+                    break
+
+        if not commit_found and pkg_commit and commits[0]['commit'] == '(null)':
+            commits[0]['commit'] = pkg_commit
 
         return PackageHistory(pkg=pkg, history=commits, pkg_status_idx=status_idx)
 
-    def install(self, pkg: FlatpakApplication, root_password: str, disk_loader: DiskCacheLoader, watcher: ProcessWatcher) -> TransactionResult:
-        config = read_config()
+    def _make_exports_dir(self, watcher: ProcessWatcher) -> bool:
+        if not os.path.exists(EXPORTS_PATH):
+            self.logger.info("Creating dir '{}'".format(EXPORTS_PATH))
+            watcher.print('Creating dir {}'.format(EXPORTS_PATH))
+            try:
+                Path(EXPORTS_PATH).mkdir(parents=True, exist_ok=True)
+            except:
+                watcher.print('Error while creating the directory {}'.format(EXPORTS_PATH))
+                return False
 
-        install_level = config['installation_level']
+        return True
+
+    def install(self, pkg: FlatpakApplication, root_password: str, disk_loader: DiskCacheLoader, watcher: ProcessWatcher) -> TransactionResult:
+        flatpak_config = self.configman.get_config()
+
+        install_level = flatpak_config['installation_level']
 
         if install_level is not None:
             self.logger.info("Default Flaptak installation level defined: {}".format(install_level))
@@ -331,12 +373,12 @@ class FlatpakManager(SoftwareManager):
             if user.is_root():
                 handler.handle_simple(flatpak.set_default_remotes('system'))
             else:
-                user_password, valid = watcher.request_root_password()
+                valid, user_password = watcher.request_root_password()
                 if not valid:
                     watcher.print('Operation aborted')
                     return TransactionResult(success=False, installed=[], removed=[])
                 else:
-                    if not handler.handle_simple(flatpak.set_default_remotes('system', user_password)):
+                    if not handler.handle_simple(flatpak.set_default_remotes('system', user_password))[0]:
                         watcher.show_message(title=self.i18n['error'].capitalize(),
                                              body=self.i18n['flatpak.remotes.system_flathub.error'],
                                              type_=MessageType.ERROR)
@@ -348,9 +390,33 @@ class FlatpakManager(SoftwareManager):
         installed = flatpak.list_installed(flatpak_version)
         installed_by_level = {'{}:{}:{}'.format(p['id'], p['name'], p['branch']) for p in installed if p['installation'] == pkg.installation} if installed else None
 
-        res = handler.handle(SystemProcess(subproc=flatpak.install(str(pkg.id), pkg.origin, pkg.installation), wrong_error_phrase='Warning'))
+        if not self._make_exports_dir(handler.watcher):
+            return TransactionResult(success=False, installed=[], removed=[])
 
-        if res:
+        installed, output = handler.handle_simple(flatpak.install(str(pkg.id), pkg.origin, pkg.installation))
+
+        if not installed and 'error: No ref chosen to resolve matches' in output:
+            ref_opts = RE_INSTALL_REFS.findall(output)
+
+            if ref_opts and len(ref_opts) > 1:
+                view_opts = [InputOption(label=o, value=o.strip()) for o in ref_opts if o]
+                ref_select = SingleSelectComponent(type_=SelectViewType.RADIO, options=view_opts, default_option=view_opts[0], label='')
+                if watcher.request_confirmation(title=self.i18n['flatpak.install.ref_choose.title'],
+                                                body=self.i18n['flatpak.install.ref_choose.body'].format(bold(pkg.name)),
+                                                components=[ref_select],
+                                                confirmation_label=self.i18n['proceed'].capitalize(),
+                                                deny_label=self.i18n['cancel'].capitalize()):
+                    ref = ref_select.get_selected()
+                    installed, output = handler.handle_simple(flatpak.install(ref, pkg.origin, pkg.installation))
+                    pkg.ref = ref
+                    pkg.runtime = 'runtime' in ref
+                else:
+                    watcher.print('Aborted by the user')
+                    return TransactionResult.fail()
+            else:
+                return TransactionResult.fail()
+
+        if installed:
             try:
                 fields = flatpak.get_fields(str(pkg.id), pkg.branch, ['Ref', 'Branch'])
 
@@ -360,21 +426,21 @@ class FlatpakManager(SoftwareManager):
             except:
                 traceback.print_exc()
 
-        if res:
+        if installed:
             new_installed = [pkg]
             current_installed = flatpak.list_installed(flatpak_version)
             current_installed_by_level = [p for p in current_installed if p['installation'] == pkg.installation] if current_installed else None
 
             if current_installed_by_level and (not installed_by_level or len(current_installed_by_level) > len(installed_by_level) + 1):
                 pkg_key = '{}:{}:{}'.format(pkg.id, pkg.name, pkg.branch)
-                net_available = internet.is_available()
+                net_available = self.context.is_internet_available()
                 for p in current_installed_by_level:
                     current_key = '{}:{}:{}'.format(p['id'], p['name'], p['branch'])
                     if current_key != pkg_key and (not installed_by_level or current_key not in installed_by_level):
                         new_installed.append(self._map_to_model(app_json=p, installed=True,
                                                                 disk_loader=disk_loader, internet=net_available))
 
-            return TransactionResult(success=res, installed=new_installed, removed=[])
+            return TransactionResult(success=installed, installed=new_installed, removed=[])
         else:
             return TransactionResult.fail()
 
@@ -387,11 +453,12 @@ class FlatpakManager(SoftwareManager):
     def can_work(self) -> bool:
         return flatpak.is_installed()
 
-    def requires_root(self, action: str, pkg: FlatpakApplication):
-        return action == 'downgrade' and pkg.installation == 'system'
+    def requires_root(self, action: SoftwareAction, pkg: FlatpakApplication) -> bool:
+        return action == SoftwareAction.DOWNGRADE and pkg.installation == 'system'
 
     def prepare(self, task_manager: TaskManager, root_password: str, internet_available: bool):
-        Thread(target=read_config, args=(True,), daemon=True).start()
+        CreateConfigFile(taskman=task_manager, configman=self.configman, i18n=self.i18n,
+                         task_icon_path=get_icon_path(), logger=self.logger).start()
 
     def list_updates(self, internet_available: bool) -> List[PackageUpdate]:
         updates = []
@@ -493,7 +560,7 @@ class FlatpakManager(SoftwareManager):
     def get_settings(self, screen_width: int, screen_height: int) -> ViewComponent:
         fields = []
 
-        config = read_config()
+        flatpak_config = self.configman.get_config()
 
         install_opts = [InputOption(label=self.i18n['flatpak.config.install_level.system'].capitalize(),
                                     value='system',
@@ -506,19 +573,19 @@ class FlatpakManager(SoftwareManager):
                                     tooltip=self.i18n['flatpak.config.install_level.ask.tip'].format(app=self.context.app_name))]
         fields.append(SingleSelectComponent(label=self.i18n['flatpak.config.install_level'],
                                             options=install_opts,
-                                            default_option=[o for o in install_opts if o.value == config['installation_level']][0],
+                                            default_option=[o for o in install_opts if o.value == flatpak_config['installation_level']][0],
                                             max_per_line=len(install_opts),
                                             max_width=floor(screen_width * 0.22),
                                             type_=SelectViewType.RADIO))
 
         return PanelComponent([FormComponent(fields, self.i18n['installation'].capitalize())])
 
-    def save_settings(self, component: PanelComponent) -> Tuple[bool, List[str]]:
-        config = read_config()
-        config['installation_level'] = component.components[0].components[0].get_selected()
+    def save_settings(self, component: PanelComponent) -> Tuple[bool, Optional[List[str]]]:
+        flatpak_config = self.configman.get_config()
+        flatpak_config['installation_level'] = component.components[0].components[0].get_selected()
 
         try:
-            save_config(config, CONFIG_FILE)
+            self.configman.save_config(flatpak_config)
             return True, None
         except:
             return False, [traceback.format_exc()]
@@ -563,8 +630,9 @@ class FlatpakManager(SoftwareManager):
             all_runtimes = []
             for runtime in runtimes:
                 for partial in partials:
-                    if partial.base_id == runtime.id:
+                    if partial.installation == runtime.installation and partial.base_id == runtime.id:
                         all_runtimes.append(partial)
+                        break
 
                 all_runtimes.append(runtime)
             return [*all_runtimes, *apps]
