@@ -1,11 +1,15 @@
-from typing import List
+from typing import List, Optional
 
 from bauh.commons.system import SimpleProcess, run_cmd
 from bauh.gems.web import NATIVEFIER_BIN_PATH, NODE_PATHS
 
 
-def install(url: str, name: str, output_dir: str, electron_version: str, cwd: str, system: bool, extra_options: List[str] = None) -> SimpleProcess:
-    cmd = [NATIVEFIER_BIN_PATH if not system else 'nativefier', url, '--name', name, '-e', electron_version, output_dir]
+def install(url: str, name: str, output_dir: str, electron_version: Optional[str], cwd: str, system: bool, extra_options: List[str] = None) -> SimpleProcess:
+    cmd = [NATIVEFIER_BIN_PATH if not system else 'nativefier', url, '--name', name, output_dir]
+
+    if electron_version:
+        cmd.append('-e')
+        cmd.append(electron_version)
 
     if extra_options:
         cmd.extend(extra_options)
