@@ -1,4 +1,4 @@
-import operator
+import re
 import re
 import time
 import traceback
@@ -7,20 +7,21 @@ from threading import Thread
 from typing import List, Set, Type, Tuple, Dict
 
 from bauh.api.abstract.controller import SoftwareManager, SearchResult, ApplicationContext, UpgradeRequirements, \
-    UpgradeRequirement, TransactionResult
+    UpgradeRequirement, TransactionResult, SoftwareAction
 from bauh.api.abstract.disk import DiskCacheLoader
 from bauh.api.abstract.handler import ProcessWatcher, TaskManager
 from bauh.api.abstract.model import SoftwarePackage, PackageUpdate, PackageHistory, PackageSuggestion, \
     CustomSoftwareAction
 from bauh.api.abstract.view import ViewComponent, TabGroupComponent, MessageType
 from bauh.api.exception import NoInternetException
-from bauh.commons import internet
+from bauh.commons.boot import CreateConfigFile
 from bauh.commons.html import bold
 from bauh.commons.system import run_cmd
-from bauh.view.core.config import read_config
+from bauh.view.core.config import CoreConfigManager
 from bauh.view.core.settings import GenericSettingsManager
 from bauh.view.core.update import check_for_update
 from bauh.view.util import resource
+from bauh.view.util.resource import get_path
 from bauh.view.util.util import clean_app_files, restart_app
 
 RE_IS_URL = re.compile(r'^https?://.+')
@@ -53,14 +54,15 @@ class GenericSoftwareManager(SoftwareManager):
         self.config = config
         self.settings_manager = settings_manager
         self.http_client = context.http_client
-        self.extra_actions = [CustomSoftwareAction(i18_label_key='action.reset',
+        self.configman = CoreConfigManager()
+        self.extra_actions = [CustomSoftwareAction(i18n_label_key='action.reset',
                                                    i18n_status_key='action.reset.status',
                                                    manager_method='reset',
                                                    manager=self,
                                                    icon_path=resource.get_path('img/logo.svg'),
                                                    requires_root=False,
                                                    refresh=False)]
-        self.dynamic_extra_actions = {CustomSoftwareAction(i18_label_key='action.backups',
+        self.dynamic_extra_actions = {CustomSoftwareAction(i18n_label_key='action.backups',
                                                            i18n_status_key='action.backups.status',
                                                            manager_method='launch_timeshift',
                                                            manager=self,
@@ -147,19 +149,19 @@ class GenericSoftwareManager(SoftwareManager):
             mti = time.time()
             apps_found = man.search(words=word, disk_loader=disk_loader, is_url=is_url)
             mtf = time.time()
-            self.logger.info(man.__class__.__name__ + " took {0:.2f} seconds".format(mtf - mti))
+            self.logger.info(man.__class__.__name__ + " took {0:.8f} seconds".format(mtf - mti))
 
             res.installed.extend(apps_found.installed)
             res.new.extend(apps_found.new)
 
-    def search(self, word: str, disk_loader: DiskCacheLoader = None, limit: int = -1, is_url: bool = False) -> SearchResult:
+    def search(self, words: str, disk_loader: DiskCacheLoader = None, limit: int = -1, is_url: bool = False) -> SearchResult:
         ti = time.time()
         self._wait_to_be_ready()
 
-        res = SearchResult([], [], 0)
+        res = SearchResult.empty()
 
-        if internet.is_available():
-            norm_word = word.strip().lower()
+        if self.context.is_internet_available():
+            norm_word = words.strip().lower()
 
             url_words = RE_IS_URL.match(norm_word)
             disk_loader = self.disk_loader_factory.new()
@@ -181,12 +183,12 @@ class GenericSoftwareManager(SoftwareManager):
 
             res.installed = self._sort(res.installed, norm_word)
             res.new = self._sort(res.new, norm_word)
-            res.total = len(res.installed) + len(res.new)
         else:
             raise NoInternetException()
 
+        res.update_total()
         tf = time.time()
-        self.logger.info('Took {0:.2f} seconds'.format(tf - ti))
+        self.logger.info('Took {0:.8f} seconds'.format(tf - ti))
         return res
 
     def _wait_to_be_ready(self):
@@ -211,7 +213,7 @@ class GenericSoftwareManager(SoftwareManager):
 
         disk_loader = None
 
-        net_available = internet.is_available()
+        net_available = self.context.is_internet_available()
         if not pkg_types:  # any type
             for man in self.managers:
                 if self._can_work(man):
@@ -386,7 +388,7 @@ class GenericSoftwareManager(SoftwareManager):
             if man:
                 return man.cache_to_disk(pkg, icon_bytes=icon_bytes, only_icon=only_icon)
 
-    def requires_root(self, action: str, app: SoftwarePackage) -> bool:
+    def requires_root(self, action: SoftwareAction, app: SoftwarePackage) -> bool:
         if app is None:
             if self.managers:
                 for man in self.managers:
@@ -401,14 +403,30 @@ class GenericSoftwareManager(SoftwareManager):
                 return man.requires_root(action, app)
 
     def prepare(self, task_manager: TaskManager, root_password: str, internet_available: bool):
+        ti = time.time()
+        self.logger.info("Initializing")
+        taskman = task_manager if task_manager else TaskManager()  # empty task manager to prevent null pointers
+
+        create_config = CreateConfigFile(taskman=taskman, configman=self.configman, i18n=self.i18n,
+                                         task_icon_path=get_path('img/logo.svg'), logger=self.logger)
+        create_config.start()
+
         if self.managers:
-            internet_on = internet.is_available()
-            taskman = task_manager if task_manager else TaskManager()  # empty task manager to prevent null pointers
+            internet_on = self.context.is_internet_available()
+            prepare_tasks = []
             for man in self.managers:
                 if man not in self._already_prepared and self._can_work(man):
-                    man.prepare(taskman, root_password, internet_on)
+                    t = Thread(target=man.prepare, args=(taskman, root_password, internet_on), daemon=True)
+                    t.start()
+                    prepare_tasks.append(t)
                     self._already_prepared.append(man)
 
+            for t in prepare_tasks:
+                t.join()
+
+        tf = time.time()
+        self.logger.info("Finished. Took {0:.2f} seconds".format(tf - ti))
+
     def cache_available_managers(self):
         if self.managers:
             for man in self.managers:
@@ -420,7 +438,7 @@ class GenericSoftwareManager(SoftwareManager):
         updates = []
 
         if self.managers:
-            net_available = internet.is_available()
+            net_available = self.context.is_internet_available()
 
             for man in self.managers:
                 if self._can_work(man):
@@ -433,7 +451,7 @@ class GenericSoftwareManager(SoftwareManager):
     def list_warnings(self, internet_available: bool = None) -> List[str]:
         warnings = []
 
-        int_available = internet.is_available()
+        int_available = self.context.is_internet_available()
 
         if int_available:
             updates_msg = check_for_update(self.logger, self.http_client, self.i18n)
@@ -459,7 +477,7 @@ class GenericSoftwareManager(SoftwareManager):
             mti = time.time()
             man_sugs = man.list_suggestions(limit=limit, filter_installed=filter_installed)
             mtf = time.time()
-            self.logger.info(man.__class__.__name__ + ' took {0:.2f} seconds'.format(mtf - mti))
+            self.logger.info(man.__class__.__name__ + ' took {0:.5f} seconds'.format(mtf - mti))
 
             if man_sugs:
                 if 0 < limit < len(man_sugs):
@@ -469,7 +487,7 @@ class GenericSoftwareManager(SoftwareManager):
 
     def list_suggestions(self, limit: int, filter_installed: bool) -> List[PackageSuggestion]:
         if bool(self.config['suggestions']['enabled']):
-            if self.managers and internet.is_available():
+            if self.managers and self.context.is_internet_available():
                 suggestions, threads = [], []
                 for man in self.managers:
                     t = Thread(target=self._fill_suggestions, args=(suggestions, man, int(self.config['suggestions']['by_type']), filter_installed))
@@ -486,6 +504,9 @@ class GenericSoftwareManager(SoftwareManager):
         return []
 
     def execute_custom_action(self, action: CustomSoftwareAction, pkg: SoftwarePackage, root_password: str, watcher: ProcessWatcher):
+        if action.requires_internet and not self.context.is_internet_available():
+            raise NoInternetException()
+
         man = action.manager if action.manager else self._get_manager_for(pkg)
 
         if man:
@@ -518,7 +539,8 @@ class GenericSoftwareManager(SoftwareManager):
                                                            working_managers=self.working_managers,
                                                            logger=self.logger,
                                                            i18n=self.i18n,
-                                                           file_downloader=self.context.file_downloader)
+                                                           file_downloader=self.context.file_downloader,
+                                                           configman=self.configman)
         else:
             self.settings_manager.managers = self.managers
             self.settings_manager.working_managers = self.working_managers
@@ -611,7 +633,7 @@ class GenericSoftwareManager(SoftwareManager):
                     if man_actions:
                         actions.extend(man_actions)
 
-        app_config = read_config()
+        app_config = self.configman.get_config()
 
         for action, available in self.dynamic_extra_actions.items():
             if available(app_config):
