@@ -3,18 +3,15 @@ import traceback
 from pathlib import Path
 from typing import Tuple, Dict, Optional, Set
 
-from PyQt5.QtCore import Qt, QSize, QTimer
-from PyQt5.QtGui import QIcon, QPixmap, QIntValidator, QCursor
+from PyQt5.QtCore import Qt, QTimer
+from PyQt5.QtGui import QIcon, QIntValidator, QCursor, QFocusEvent
 from PyQt5.QtWidgets import QRadioButton, QGroupBox, QCheckBox, QComboBox, QGridLayout, QWidget, \
     QLabel, QSizePolicy, QLineEdit, QToolButton, QHBoxLayout, QFormLayout, QFileDialog, QTabWidget, QVBoxLayout, \
-    QSlider, QScrollArea, QFrame, QAction, QSpinBox
+    QSlider, QScrollArea, QFrame, QAction, QSpinBox, QPlainTextEdit, QWidgetAction, QPushButton, QMenu
 
 from bauh.api.abstract.view import SingleSelectComponent, InputOption, MultipleSelectComponent, SelectViewType, \
     TextInputComponent, FormComponent, FileChooserComponent, ViewComponent, TabGroupComponent, PanelComponent, \
-    TwoStateButtonComponent, TextComponent, SpacerComponent, RangeInputComponent
-from bauh.view.qt import css
-from bauh.view.qt.colors import RED
-from bauh.view.util import resource
+    TwoStateButtonComponent, TextComponent, SpacerComponent, RangeInputComponent, ViewObserver, TextInputType
 from bauh.view.util.translation import I18n
 
 
@@ -26,7 +23,7 @@ class QtComponentsManager:
         self.group_of_groups = {}
         self._saved_states = {}
 
-    def register_component(self, component_id: int, instance: QWidget, action: QAction = None):
+    def register_component(self, component_id: int, instance: QWidget, action: Optional[QAction] = None):
         comp = (instance, action, {'v': True, 'e': True, 'r': False})
         self.components[component_id] = comp
         self._save_state(comp)
@@ -266,9 +263,13 @@ class RadioButtonQt(QRadioButton):
         self.model = model
         self.model_parent = model_parent
         self.toggled.connect(self._set_checked)
+        self.setCursor(QCursor(Qt.PointingHandCursor))
 
         if model.icon_path:
-            self.setIcon(QIcon(model.icon_path))
+            if model.icon_path.startswith('/'):
+                self.setIcon(QIcon(model.icon_path))
+            else:
+                self.setIcon(QIcon.fromTheme(model.icon_path))
 
         if self.model.read_only:
             self.setAttribute(Qt.WA_TransparentForMouseEvents)
@@ -291,11 +292,16 @@ class CheckboxQt(QCheckBox):
         self.setToolTip(model.tooltip)
 
         if model.icon_path:
-            self.setIcon(QIcon(model.icon_path))
+            if model.icon_path.startswith('/'):
+                self.setIcon(QIcon(model.icon_path))
+            else:
+                self.setIcon(QIcon.fromTheme(model.icon_path))
 
         if model.read_only:
             self.setAttribute(Qt.WA_TransparentForMouseEvents)
             self.setFocusPolicy(Qt.NoFocus)
+        else:
+            self.setCursor(QCursor(Qt.PointingHandCursor))
 
     def _set_checked(self, state):
         checked = state == 2
@@ -330,6 +336,8 @@ class FormComboBoxQt(QComboBox):
     def __init__(self, model: SingleSelectComponent):
         super(FormComboBoxQt, self).__init__()
         self.model = model
+        self.setCursor(QCursor(Qt.PointingHandCursor))
+        self.view().setCursor(QCursor(Qt.PointingHandCursor))
 
         if model.max_width > 0:
             self.setMaximumWidth(model.max_width)
@@ -392,8 +400,10 @@ class RadioSelectQt(QGroupBox):
 
     def __init__(self, model: SingleSelectComponent):
         super(RadioSelectQt, self).__init__(model.label + ' :' if model.label else None)
+        if not model.label:
+            self.setObjectName('radio_select_notitle')
+
         self.model = model
-        self.setStyleSheet("QGroupBox { font-weight: bold }")
 
         grid = QGridLayout()
         self.setLayout(grid)
@@ -422,10 +432,36 @@ class ComboSelectQt(QGroupBox):
     def __init__(self, model: SingleSelectComponent):
         super(ComboSelectQt, self).__init__()
         self.model = model
-        self.setLayout(QGridLayout())
-        self.setStyleSheet('QGridLayout {margin-left: 0} QLabel { font-weight: bold}')
-        self.layout().addWidget(QLabel(model.label + ' :' if model.label else ''), 0, 0)
-        self.layout().addWidget(FormComboBoxQt(model), 0, 1)
+        self._layout = QGridLayout()
+        self.setLayout(self._layout)
+        self._layout.addWidget(QLabel(model.label + ' :' if model.label else ''), 0, 0)
+        self._layout.addWidget(FormComboBoxQt(model), 0, 1)
+
+
+class QLineEditObserver(QLineEdit, ViewObserver):
+
+    def __init__(self, **kwargs):
+        super(QLineEditObserver, self).__init__(**kwargs)
+
+    def on_change(self, change: str):
+        if self.text() != change:
+            self.setText(change if change is not None else '')
+
+
+class QPlainTextEditObserver(QPlainTextEdit, ViewObserver):
+
+    def __init__(self, **kwargs):
+        super(QPlainTextEditObserver, self).__init__(**kwargs)
+
+    def on_change(self, change: str):
+        self.setText(change)
+
+    def setText(self, text: str):
+        if text != self.toPlainText():
+            self.setPlainText(text if text is not None else '')
+
+    def setCursorPosition(self, idx: int):
+        self.textCursor().setPosition(idx)
 
 
 class TextInputQt(QGroupBox):
@@ -434,13 +470,11 @@ class TextInputQt(QGroupBox):
         super(TextInputQt, self).__init__()
         self.model = model
         self.setLayout(QGridLayout())
-        self.setStyleSheet('QGridLayout {margin-left: 0} QLabel { font-weight: bold}')
-        self.layout().addWidget(QLabel(model.label.capitalize() + ' :' if model.label else ''), 0, 0)
 
         if self.model.max_width > 0:
             self.setMaximumWidth(self.model.max_width)
 
-        self.text_input = QLineEdit()
+        self.text_input = QLineEditObserver() if model.type == TextInputType.SINGLE_LINE else QPlainTextEditObserver()
 
         if model.only_int:
             self.text_input.setValidator(QIntValidator())
@@ -448,26 +482,33 @@ class TextInputQt(QGroupBox):
         if model.placeholder:
             self.text_input.setPlaceholderText(model.placeholder)
 
+        if model.min_width >= 0:
+            self.text_input.setMinimumWidth(model.min_width)
+
+        if model.min_height >= 0:
+            self.text_input.setMinimumHeight(model.min_height)
+
         if model.tooltip:
             self.text_input.setToolTip(model.tooltip)
 
-        if model.value:
+        if model.value is not None:
             self.text_input.setText(model.value)
             self.text_input.setCursorPosition(0)
 
         self.text_input.textChanged.connect(self._update_model)
 
+        self.model.observers.append(self.text_input)
         self.layout().addWidget(self.text_input, 0, 1)
 
-    def _update_model(self, text: str):
-        self.model.value = text
+    def _update_model(self, *args):
+        change = args[0] if args else self.text_input.toPlainText()
+        self.model.set_value(val=change, caller=self)
 
 
 class MultipleSelectQt(QGroupBox):
 
     def __init__(self, model: MultipleSelectComponent, callback):
         super(MultipleSelectQt, self).__init__(model.label if model.label else None)
-        self.setStyleSheet(css.GROUP_BOX)
         self.model = model
         self._layout = QGridLayout()
         self.setLayout(self._layout)
@@ -487,17 +528,6 @@ class MultipleSelectQt(QGroupBox):
 
         col = 0
 
-        pixmap_help = QPixmap()
-
-        for op in model.options:  # loads the help icon if at least one option has a tooltip
-            if op.tooltip:
-                try:
-                    pixmap_help = QIcon(resource.get_path('img/about.svg')).pixmap(QSize(16, 16))
-                except:
-                    traceback.print_exc()
-
-                break
-
         for op in model.options:
             comp = CheckboxQt(op, model, callback)
 
@@ -511,7 +541,8 @@ class MultipleSelectQt(QGroupBox):
 
             if op.tooltip:
                 help_icon = QLabel()
-                help_icon.setPixmap(pixmap_help)
+                help_icon.setProperty('help_icon', 'true')
+                help_icon.setCursor(QCursor(Qt.WhatsThisCursor))
                 help_icon.setToolTip(op.tooltip)
                 widget.layout().addWidget(help_icon)
 
@@ -546,23 +577,12 @@ class FormMultipleSelectQt(QWidget):
 
         if model.label:
             line = 1
-            self.layout().addWidget(QLabel(), 0, 1)
+            self._layout.addWidget(QLabel(), 0, 1)
         else:
             line = 0
 
         col = 0
 
-        pixmap_help = QPixmap()
-
-        for op in model.options:  # loads the help icon if at least one option has a tooltip
-            if op.tooltip:
-                try:
-                    pixmap_help = QIcon(resource.get_path('img/about.svg')).pixmap(QSize(16, 16))
-                except:
-                    traceback.print_exc()
-
-                break
-
         for op in model.options:
             comp = CheckboxQt(op, model, None)
 
@@ -576,8 +596,9 @@ class FormMultipleSelectQt(QWidget):
 
             if op.tooltip:
                 help_icon = QLabel()
-                help_icon.setPixmap(pixmap_help)
+                help_icon.setProperty('help_icon', 'true')
                 help_icon.setToolTip(op.tooltip)
+                help_icon.setCursor(QCursor(Qt.WhatsThisCursor))
                 widget.layout().addWidget(help_icon)
 
             self._layout.addWidget(widget, line, col)
@@ -614,7 +635,7 @@ class InputFilter(QLineEdit):
         if self.typing.isActive():
             return
 
-        self.typing.start(2000)
+        self.typing.start(3000)
 
     def get_text(self):
         return self.last_text
@@ -624,40 +645,26 @@ class InputFilter(QLineEdit):
         self.last_text = p_str
 
 
-class IconButton(QWidget):
+class IconButton(QToolButton):
 
-    def __init__(self, icon: QIcon, action, i18n: I18n, background: str = None, align: int = Qt.AlignCenter, tooltip: str = None, expanding: bool = False):
+    def __init__(self, action, i18n: I18n, align: int = Qt.AlignCenter, tooltip: str = None, expanding: bool = False):
         super(IconButton, self).__init__()
-        self.bt = QToolButton()
-        self.bt.setCursor(QCursor(Qt.PointingHandCursor))
-        self.bt.setIcon(icon)
-        self.bt.clicked.connect(action)
+        self.setCursor(QCursor(Qt.PointingHandCursor))
+        self.clicked.connect(action)
         self.i18n = i18n
         self.default_tootip = tooltip
-        self.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
-        self.bt.setSizePolicy(QSizePolicy.Expanding if expanding else QSizePolicy.Minimum, QSizePolicy.Minimum)
-
-        if background:
-            style = 'QToolButton { color: white; background: ' + background + '} '
-            style += 'QToolButton:disabled { color: white; background: blue }'
-            self.bt.setStyleSheet(style)
+        self.setSizePolicy(QSizePolicy.Expanding if expanding else QSizePolicy.Minimum, QSizePolicy.Minimum)
 
         if tooltip:
-            self.bt.setToolTip(tooltip)
-
-        layout = QHBoxLayout()
-        layout.setContentsMargins(0, 0, 0, 0)
-        layout.setAlignment(align)
-        layout.addWidget(self.bt)
-        self.setLayout(layout)
+            self.setToolTip(tooltip)
 
     def setEnabled(self, enabled):
         super(IconButton, self).setEnabled(enabled)
 
         if not enabled:
-            self.bt.setToolTip(self.i18n['icon_button.tooltip.disabled'])
+            self.setToolTip(self.i18n['icon_button.tooltip.disabled'])
         else:
-            self.bt.setToolTip(self.default_tootip)
+            self.setToolTip(self.default_tootip)
 
 
 class PanelQt(QWidget):
@@ -681,40 +688,50 @@ class FormQt(QGroupBox):
         self.model = model
         self.i18n = i18n
         self.setLayout(QFormLayout())
-        self.setStyleSheet(css.GROUP_BOX)
 
         if model.spaces:
             self.layout().addRow(QLabel(), QLabel())
 
-        for c in model.components:
+        for idx, c in enumerate(model.components):
             if isinstance(c, TextInputComponent):
                 label, field = self._new_text_input(c)
                 self.layout().addRow(label, field)
             elif isinstance(c, SingleSelectComponent):
                 label = self._new_label(c)
-                field = FormComboBoxQt(c) if c.type == SelectViewType.COMBO else FormRadioSelectQt(c)
-                self.layout().addRow(label, self._wrap(field, c))
+                form = FormComboBoxQt(c) if c.type == SelectViewType.COMBO else FormRadioSelectQt(c)
+                field = self._wrap(form, c)
+                self.layout().addRow(label, field)
             elif isinstance(c, RangeInputComponent):
                 label = self._new_label(c)
-                self.layout().addRow(label, self._wrap(self._new_range_input(c), c))
+                field = self._wrap(self._new_range_input(c), c)
+                self.layout().addRow(label, field)
             elif isinstance(c, FileChooserComponent):
                 label, field = self._new_file_chooser(c)
                 self.layout().addRow(label, field)
             elif isinstance(c, FormComponent):
-                self.layout().addRow(FormQt(c, self.i18n))
+                label, field = None,  FormQt(c, self.i18n)
+                self.layout().addRow(field)
             elif isinstance(c, TwoStateButtonComponent):
-                label = self._new_label(c)
-                self.layout().addRow(label, TwoStateButtonQt(c))
+                label, field = self._new_label(c), TwoStateButtonQt(c)
+                self.layout().addRow(label, field)
             elif isinstance(c, MultipleSelectComponent):
-                label = self._new_label(c)
-                self.layout().addRow(label, FormMultipleSelectQt(c))
+                label, field = self._new_label(c), FormMultipleSelectQt(c)
+                self.layout().addRow(label, field)
             elif isinstance(c, TextComponent):
-                self.layout().addRow(self._new_label(c), QWidget())
+                label, field = self._new_label(c), QWidget()
+                self.layout().addRow(label, field)
             elif isinstance(c, RangeInputComponent):
-                self.layout()
+                label, field = self._new_label(c), self._new_range_input(c)
+                self.layout().addRow(label, field)
             else:
                 raise Exception('Unsupported component type {}'.format(c.__class__.__name__))
 
+            if label:  # to prevent C++ wrap errors
+                setattr(self, 'label_{}'.format(idx), label)
+
+            if field:  # to prevent C++ wrap errors
+                setattr(self, 'field_{}'.format(idx), field)
+
         if model.spaces:
             self.layout().addRow(QLabel(), QLabel())
 
@@ -728,8 +745,11 @@ class FormQt(QGroupBox):
         if hasattr(comp, 'size') and comp.size is not None:
             label_comp.setStyleSheet("QLabel { font-size: " + str(comp.size) + "px }")
 
-        attr = 'label' if hasattr(comp,'label') else 'value'
-        text = getattr(comp, attr)
+        if hasattr(comp, 'get_label'):
+            text = comp.get_label()
+        else:
+            attr = 'label' if hasattr(comp, 'label') else 'value'
+            text = getattr(comp, attr)
 
         if text:
             if hasattr(comp, 'capitalize_label') and getattr(comp, 'capitalize_label'):
@@ -744,38 +764,41 @@ class FormQt(QGroupBox):
 
     def gen_tip_icon(self, tip: str) -> QLabel:
         tip_icon = QLabel()
+        tip_icon.setProperty('tip_icon', 'true')
         tip_icon.setToolTip(tip.strip())
-
-        try:
-            tip_icon.setPixmap(QIcon(resource.get_path('img/about.svg')).pixmap(QSize(12, 12)))
-        except:
-            traceback.print_exc()
-
+        tip_icon.setCursor(QCursor(Qt.WhatsThisCursor))
         return tip_icon
 
     def _new_text_input(self, c: TextInputComponent) -> Tuple[QLabel, QLineEdit]:
-        line_edit = QLineEdit()
+        view = QLineEditObserver() if c.type == TextInputType.SINGLE_LINE else QPlainTextEditObserver()
+
+        if c.min_width >= 0:
+            view.setMinimumWidth(c.min_width)
+
+        if c.min_height >= 0:
+            view.setMinimumHeight(c.min_height)
 
         if c.only_int:
-            line_edit.setValidator(QIntValidator())
+            view.setValidator(QIntValidator())
 
         if c.tooltip:
-            line_edit.setToolTip(c.tooltip)
+            view.setToolTip(c.tooltip)
 
         if c.placeholder:
-            line_edit.setPlaceholderText(c.placeholder)
+            view.setPlaceholderText(c.placeholder)
 
-        if c.value:
-            line_edit.setText(str(c.value) if c.value else '')
-            line_edit.setCursorPosition(0)
+        if c.value is not None:
+            view.setText(str(c.value))
+            view.setCursorPosition(0)
 
         if c.read_only:
-            line_edit.setEnabled(False)
+            view.setEnabled(False)
 
         def update_model(text: str):
-            c.value = text
+            c.set_value(val=text, caller=view)
 
-        line_edit.textChanged.connect(update_model)
+        view.textChanged.connect(update_model)
+        c.observers.append(view)
 
         label = QWidget()
         label.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
@@ -785,15 +808,16 @@ class FormQt(QGroupBox):
         label.layout().addWidget(label_component)
 
         if label:
-            label_component.setText(c.label.capitalize())
+            label_component.setText(c.get_label())
 
             if c.tooltip:
                 label.layout().addWidget(self.gen_tip_icon(c.tooltip))
 
-        return label, self._wrap(line_edit, c)
+        return label, self._wrap(view, c)
 
     def _new_range_input(self, model: RangeInputComponent) -> QSpinBox:
         spinner = QSpinBox()
+        spinner.setCursor(QCursor(Qt.PointingHandCursor))
         spinner.setMinimum(model.min)
         spinner.setMaximum(model.max)
         spinner.setSingleStep(model.step)
@@ -819,7 +843,7 @@ class FormQt(QGroupBox):
         return field_container
 
     def _new_file_chooser(self, c: FileChooserComponent) -> Tuple[QLabel, QLineEdit]:
-        chooser = QLineEdit()
+        chooser = QLineEditObserver()
         chooser.setReadOnly(True)
 
         if c.max_width > 0:
@@ -827,46 +851,44 @@ class FormQt(QGroupBox):
 
         if c.file_path:
             chooser.setText(c.file_path)
+            chooser.setCursorPosition(0)
 
+        c.observers.append(chooser)
         chooser.setPlaceholderText(self.i18n['view.components.file_chooser.placeholder'])
 
         def open_chooser(e):
-            options = QFileDialog.Options()
-
             if c.allowed_extensions:
                 exts = ';;'.join({'*.{}'.format(e) for e in c.allowed_extensions})
             else:
-                exts = '{}} (*);;'.format(self.i18n['all_files'].capitalize())
+                exts = '{} (*);;'.format(self.i18n['all_files'].capitalize())
 
             if c.file_path and os.path.isfile(c.file_path):
                 cur_path = c.file_path
+            elif c.search_path and os.path.exists(c.search_path):
+                cur_path = c.search_path
             else:
                 cur_path = str(Path.home())
 
-            file_path, _ = QFileDialog.getOpenFileName(self, self.i18n['file_chooser.title'], cur_path, exts, options=options)
+            if c.directory:
+                file_path = QFileDialog.getExistingDirectory(self, self.i18n['file_chooser.title'], cur_path, options=QFileDialog.Options())
+            else:
+                file_path, _ = QFileDialog.getOpenFileName(self, self.i18n['file_chooser.title'], cur_path, exts, options=QFileDialog.Options())
 
             if file_path:
-                c.file_path = file_path
-                chooser.setText(file_path)
+                c.set_file_path(file_path)
 
             chooser.setCursorPosition(0)
 
         def clean_path():
-            c.file_path = None
-            chooser.setText('')
+            c.set_file_path(None)
 
         chooser.mousePressEvent = open_chooser
 
         label = self._new_label(c)
         wrapped = self._wrap(chooser, c)
 
-        try:
-            icon = QIcon(resource.get_path('img/clean.svg'))
-        except:
-            traceback.print_exc()
-            icon = QIcon()
-
-        bt = IconButton(icon, i18n=self.i18n['clean'].capitalize(), action=clean_path, background=RED, tooltip=self.i18n['action.run.tooltip'])
+        bt = IconButton(i18n=self.i18n['clean'].capitalize(), action=clean_path, tooltip=self.i18n['clean'].capitalize())
+        bt.setObjectName('clean_field')
 
         wrapped.layout().addWidget(bt)
         return label, wrapped
@@ -893,6 +915,8 @@ class TabGroupQt(QTabWidget):
             scroll.setWidget(to_widget(c.content, i18n))
             self.addTab(scroll, icon, c.label)
 
+        self.tabBar().setCursor(QCursor(Qt.PointingHandCursor))
+
 
 def new_single_select(model: SingleSelectComponent) -> QWidget:
     if model.type == SelectViewType.RADIO:
@@ -905,6 +929,7 @@ def new_single_select(model: SingleSelectComponent) -> QWidget:
 
 def new_spacer(min_width: int = None) -> QWidget:
     spacer = QWidget()
+    spacer.setProperty('spacer', 'true')
 
     if min_width:
         spacer.setMinimumWidth(min_width)
@@ -950,13 +975,13 @@ class RangeInputQt(QGroupBox):
         super(RangeInputQt, self).__init__()
         self.model = model
         self.setLayout(QGridLayout())
-        self.setStyleSheet('QGridLayout {margin-left: 0} QLabel { font-weight: bold}')
         self.layout().addWidget(QLabel(model.label.capitalize() + ' :' if model.label else ''), 0, 0)
 
         if self.model.max_width > 0:
             self.setMaximumWidth(self.model.max_width)
 
         self.spinner = QSpinBox()
+        self.spinner.setCursor(QCursor(Qt.PointingHandCursor))
         self.spinner.setMinimum(model.min)
         self.spinner.setMaximum(model.max)
         self.spinner.setSingleStep(model.step)
@@ -971,3 +996,157 @@ class RangeInputQt(QGroupBox):
 
     def _update_value(self):
         self.model.value = self.spinner.value()
+
+
+class QCustomLineEdit(QLineEdit):
+
+    def __init__(self, focus_in_callback, focus_out_callback, **kwargs):
+        super(QCustomLineEdit, self).__init__(**kwargs)
+        self.focus_in_callback = focus_in_callback
+        self.focus_out_callback = focus_out_callback
+
+    def focusInEvent(self, ev: QFocusEvent):
+        super(QCustomLineEdit, self).focusInEvent(ev)
+        if self.focus_in_callback:
+            self.focus_in_callback()
+
+    def focusOutEvent(self, ev: QFocusEvent):
+        super(QCustomLineEdit, self).focusOutEvent(ev)
+        if self.focus_out_callback:
+            self.focus_out_callback()
+
+        self.clearFocus()
+
+
+class QSearchBar(QWidget):
+
+    def __init__(self, search_callback, parent: Optional[QWidget] = None):
+        super(QSearchBar, self).__init__(parent=parent)
+        self.setLayout(QHBoxLayout())
+        self.setContentsMargins(0, 0, 0, 0)
+        self.layout().setSpacing(0)
+        self.callback = search_callback
+
+        self.inp_search = QCustomLineEdit(focus_in_callback=self._set_focus_in,
+                                          focus_out_callback=self._set_focus_out)
+        self.inp_search.setObjectName('inp_search')
+        self.inp_search.setFrame(False)
+        self.inp_search.returnPressed.connect(search_callback)
+        search_background_color = self.inp_search.palette().color(self.inp_search.backgroundRole()).name()
+
+        self.search_left_corner = QLabel()
+        self.search_left_corner.setObjectName('lb_left_corner')
+
+        self.layout().addWidget(self.search_left_corner)
+
+        self.layout().addWidget(self.inp_search)
+
+        self.search_button = QPushButton()
+        self.search_button.setObjectName('search_button')
+        self.search_button.setCursor(QCursor(Qt.PointingHandCursor))
+        self.search_button.clicked.connect(search_callback)
+
+        self.layout().addWidget(self.search_button)
+
+    def clear(self):
+        self.inp_search.clear()
+
+    def text(self) -> str:
+        return self.inp_search.text()
+
+    def set_text(self, text: str):
+        self.inp_search.setText(text)
+
+    def setFocus(self):
+        self.inp_search.setFocus()
+
+    def set_tooltip(self, tip: str):
+        self.inp_search.setToolTip(tip)
+
+    def set_button_tooltip(self, tip: str):
+        self.search_button.setToolTip(tip)
+
+    def set_placeholder(self, placeholder: str):
+        self.inp_search.setPlaceholderText(placeholder)
+
+    def _set_focus_in(self):
+        self.search_button.setProperty('focused', 'true')
+        self.search_left_corner.setProperty('focused', 'true')
+
+        for c in (self.search_button, self.search_left_corner):
+            c.style().unpolish(c)
+            c.style().polish(c)
+
+    def _set_focus_out(self):
+        self.search_button.setProperty('focused', 'false')
+        self.search_left_corner.setProperty('focused', 'false')
+
+        for c in (self.search_button, self.search_left_corner):
+            c.style().unpolish(c)
+            c.style().polish(c)
+
+
+class QCustomMenuAction(QWidgetAction):
+
+    def __init__(self, parent: QWidget, label: Optional[str] = None, action=None, button_name: Optional[str] = None,
+                 icon: Optional[QIcon] = None, tooltip: Optional[str] = None):
+        super(QCustomMenuAction, self).__init__(parent)
+        self.button = QPushButton()
+        self.set_label(label)
+        self._action = None
+        self.set_action(action)
+        self.set_button_name(button_name)
+        self.set_icon(icon)
+        self.setDefaultWidget(self.button)
+
+        if tooltip:
+            self.button.setToolTip(tooltip)
+
+    def set_label(self, label: str):
+        self.button.setText(label)
+
+    def set_action(self, action):
+        self._action = action
+        self.button.clicked.connect(self._handle_action)
+
+    def _handle_action(self):
+        if self._action:
+            self._action()
+
+            if self.parent() and isinstance(self.parent(), QMenu):
+                self.parent().close()
+
+    def set_button_name(self, name: str):
+        if name:
+            self.button.setObjectName(name)
+
+    def set_icon(self, icon: QIcon):
+        if icon:
+            self.button.setIcon(icon)
+
+    def get_label(self) -> str:
+        return self.button.text()
+
+
+class QCustomToolbar(QWidget):
+
+    def __init__(self, spacing: int = 2, parent: Optional[QWidget] = None, alignment: Qt.Alignment = Qt.AlignRight,
+                 policy_width: QSizePolicy.Policy = QSizePolicy.Minimum,
+                 policy_height: QSizePolicy.Policy = QSizePolicy.Preferred):
+        super(QCustomToolbar, self).__init__(parent=parent)
+        self.setProperty('container', 'true')
+        self.setSizePolicy(policy_width, policy_height)
+        self.setLayout(QHBoxLayout())
+        self.layout().setContentsMargins(0, 0, 0, 0)
+        self.layout().setSpacing(spacing)
+        self.layout().setAlignment(alignment)
+
+    def add_widget(self, widget: QWidget):
+        if widget:
+            self.layout().addWidget(widget)
+
+    def add_stretch(self, value: int = 0):
+        self.layout().addStretch(value)
+
+    def add_space(self, min_width: int = 0):
+        self.layout().addWidget(new_spacer(min_width))
